unit CoherenceTest;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ToolWin, ExtCtrls, TeeProcs, TeEngine, Chart,
  SignalToolsTee, TeeEdit, SignalToolsDialogs, SpectrumAna, MtxBaseComp,
  SignalTools, SignalAnalysis, Series, StdCtrls, TeeTools, FileSignal,
  MtxDialogs, SignalUtils;

type
  TCoherenceTestForm = class(TForm)
    SpectrumChart: TSpectrumChart;
    Panel1: TPanel;
    ToolBar1: TToolBar;
    ChartEditButton: TToolButton;
    SpectrumAnalyzerDialog: TSpectrumAnalyzerDialog;
    ChartEditor: TChartEditor;
    Series1: TFastLineSeries;
    RichEdit1: TRichEdit;
    ChartTool1: TAxisScaleTool;
    Series2: TPointSeries;
    SignalRead1: TSignalRead;
    CrossAnalyzer: TCrossSpectrumAnalyzer;
    FilteredSignal: TSignal;
    SpectrumBox: TComboBox;
    Label1: TLabel;
    procedure SpectrumEditButtonClick(Sender: TObject);
    procedure ChartEditButtonClick(Sender: TObject);
    procedure SpectrumAnalyzerParameterUpdate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SignalRead1AfterUpdate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ToolButton1Click(Sender: TObject);
    procedure SpectrumBoxChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    FirState: TFirState;
  end;

var
  CoherenceTestForm: TCoherenceTestForm;

implementation

{$R *.dfm}

uses MtxVec;

procedure TCoherenceTestForm.SpectrumEditButtonClick(Sender: TObject);
begin
     SpectrumAnalyzerDialog.Execute;
end;

procedure TCoherenceTestForm.ChartEditButtonClick(Sender: TObject);
begin
    ChartEditor.Execute;
end;

procedure TCoherenceTestForm.SpectrumAnalyzerParameterUpdate(Sender: TObject);
begin
     CrossAnalyzer.Update;
end;

procedure TCoherenceTestForm.FormCreate(Sender: TObject);
var h: TVec;
begin
     CreateIt(h);
     try
        KaiserImpulse(h,[0.2,0.25],0.001,ftLowPass,1);
        FirInit(H,FirState);
     finally
        FreeIt(h);
     end;
     SignalRead1.FileName := ExtractFileDir(Application.ExeName) + '\bz.sfs';     
     SpectrumBox.ItemIndex := 0;
     ToolButton1Click(Sender);
     With RichEdit1.Lines, RichEdit1 do
     begin
        Clear;
        Add('The cross spectral analysis is usefull when trying to determine:');
        Add('');
        Paragraph.FirstIndent := 10;
        Paragraph.Numbering := nsBullet;
        Add('What is the transfer function of the system.');
        Add('Is the system linear or not. If it is not linear, at which frequencies ' +
            'is the system non-linear.');
        Paragraph.FirstIndent := 0;
        Paragraph.Numbering := nsNone;
        Add('');
        Add('Change the value of the spectrum box to see the frequency spectrum of the signal that was' +
            'send in to the system (input spectrum), what came out (output spectrum), what is the ' +
            'frequency response of the system (transfer function) and the test for linearity (coherence). ' +
            'The system in this test case is a low pass FIR filter. With sufficient averaging, coherence ' +
            'would converge to 1. A coherence of 1 means that the system is linear at that frequency.');
        Paragraph.FirstIndent := 0;
        Paragraph.Numbering := nsNone;
    end;
end;

procedure TCoherenceTestForm.SignalRead1AfterUpdate(Sender: TObject);
begin
     FirFilter(SignalRead1.Data, FilteredSignal.Data,FirState);
     FilteredSignal.SamplingFrequency := SignalRead1.SamplingFrequency;
end;

procedure TCoherenceTestForm.FormDestroy(Sender: TObject);
begin
     FirFree(FirState);
end;

procedure TCoherenceTestForm.ToolButton1Click(Sender: TObject);
begin
     SignalRead1.RecordPosition := 0;
//   SignalRead1.OverlappingPercent := 50;  we can not use this, because of the filter
     with CrossAnalyzer do
     begin
          ResetAveraging;
          Logarithmic := True;
          Window := wtHanning;
          LogType := ltRelative;
     end;
     CrossAnalyzer.CrossAnalyzer.Recursive := false;
     while CrossAnalyzer.Pull do
     begin
          //fast enough so there is no need for a progress indicator here
     end;
     CrossAnalyzer.CrossAnalyzer.Update;  //if recursive would be True, this would not be neccessary.
     SpectrumBoxChange(Sender);
end;

procedure TCoherenceTestForm.SpectrumBoxChange(Sender: TObject);
begin
     CrossAnalyzer.CrossAnalyzer.Transform := TCrossTransform(SpectrumBox.ItemIndex);
     SpectrumChart.Title.Text.Clear;
     SpectrumChart.Title.Text.Add(CrossTransformToString(CrossAnalyzer.CrossAnalyzer.Transform));
     CrossAnalyzer.UpdateSpectrum;
end;

initialization
RegisterClass(TCoherenceTestForm);

end.
