{*******************************************}
{   Signal Tools 1.0 DSP Components	    }
{    					    }
{   Types: TChartRecord                     }
{                                           }
{   Components:  TSignalDiscreteSeries      }
{                TSignalHighLowSeries       }
{                                           }
{   Copyright (c) 1997-2003 Dew Research    }
{                 Janez Makovsek            }
{   All Rights Reserved                     }
{*******************************************}
{$I BdsppDefs.inc}
{:Additional series for TTeeChart.}
unit SignalSeriesTee;

interface

uses Windows,
     Graphics, Forms,
     TeEngine, Chart, SysUtils, Classes, TeCanvas, TeeProcs,Series,ErrorBar
     {$IFDEF D6}, Types {$ENDIF};

type

  {:Defines the drawing style of the TSignalHighLow series.
     Defines how the TSignalHighLow series will drawn.
     With ssmHighLow, the series will look like a
     regular THighLowSeries. With ssmLine, the series
     will look like a line series.}

  TSignalSeriesMode = (ssmHighLow,ssmLine);

  {:Series usefull for signal overview charts.
    Use this series for signal overview charts.
     The series can look like a THighLowSeries or like a TLineSeries.
     The user can switch between both looks with the SeriesMode property.
     Both drawing modes are very fast.}

  TSignalHighLowSeries = class(THighLowSeries)
  private
     FSeriesMode: TSignalSeriesMode;
//     procedure DrawAllValuesVec;
    procedure  SetSeriesMode(const Value: TSignalSeriesMode);
  protected
     function MoreSameZOrder: Boolean; override;
     procedure DrawAllValues; override;
  public
     {::Get origin position. Used by TChart.}
     Function MaxYValue:Double; override;
     {::Get origin position. Used by TChart.}
     Function MinYValue:Double; override;
     {::Create the component.}
     constructor Create(AOwner: TComponent); override;
     {::Assign Source to Self.}
     procedure Assign(Source: TPersistent); override;
  published
     {::Set the series drawing mode.}
     property SeriesMode: TSignalSeriesMode read FSeriesMode write SetSeriesMode;
  end;

{  TSignalAreaSeries=Class(TAreaSeries)
  private
    tmpDark3DRatio: double;
    FFrontPanelOnly: boolean;
    function AAVCalcYPosLeftRight(const YLimit: Double; AnotherIndex,
      ValueIndex: Integer): Integer;
    Procedure AAVDrawPoint(ValueIndex,i:Integer;const a: array of TChartRecord; IsLastValue: boolean);
    procedure AAVInternalDrawArea(BrushColor: TColor;
      ValueIndexi,i:Integer;const a: array of TChartRecord; IsLastValue: boolean);
    procedure DrawPointers(const a: array of TChartRecord; k: integer);
    procedure DrawLines(const a: array of TChartRecord);
    procedure SetFrontPlaneOnly(const Value: boolean);
  protected
    class Function GetEditorClass:String; override;
    procedure DrawAllValues; override;
    function MoreSameZOrder: Boolean; override;
  public
   function GetOriginPos(ValueIndex: Integer): Integer; override;
  published
    property FrontPlaneOnly: boolean read FFrontPanelOnly write SetFrontPlaneOnly;
  end;}

  {:A series drawing a discrete signal sample by sample.
    The series draws a discrete signal.}
  TSignalDiscreteSeries=class(TPointSeries)
   private
     FUseOrigin: Boolean;
     FOrigin: Double;
     FTargetZOrder: integer;
     procedure PrepareCanvas(AColor: TColor);
     procedure SetOrigin(const Value: Double);
     procedure SetUseOrigin(const Value: Boolean);
     procedure SetTargetZOrder(const Value: integer);
   protected        
     procedure DrawAllValues; override;
     procedure CalcZOrder; override;
     Function MoreSameZOrder:Boolean; override;
     class Function GetEditorClass:String; override;
   public
     {::Defines the Z-order at which the series should be drawn.}
     property TargetZOrder: integer read FTargetZOrder write SetTargetZOrder;
     {::Create the component.}
     Constructor Create(AOwner: TComponent); override;
     {::Assign the component.}
     procedure Assign(Source: TPersistent); override;
     {::Get origin position. Used by TChart.}
     function GetOriginPos(ValueIndex: Integer): Integer; override;
     {::Get maximum Y value. Used by TChart.}
     function MaxYValue: Double; override;
     {::Get minimum Y value. Used by TChart.}
     function MinYValue: Double; override;
   published
     {::If True, the value of YOrigin property will be used when determining
        the start of drawing in the Y-axis direction.}
     property UseYOrigin:Boolean read FUseOrigin write SetUseOrigin default False;
     {::Defines the start of vertical lines in the Y-axis direction.}
     property YOrigin:Double read FOrigin write SetOrigin;
   end;

implementation

{$R SignalToolsTee.res}
uses TeeConst, SignalDiscreteSeriesEdit;

{$IFDEF TEE5}
var TeeMsg_GallerySignalArea,
    TeeMsg_GallerySignalPoint,
    TeeMsg_GallerySignalHighLow,
    TeeMsg_GalleryDSP: String;
{$ELSE}
const TeeMsg_GallerySignalArea = 'Signal area';
      TeeMsg_GallerySignalPoint = 'Signal point';
      TeeMsg_GalleryDSP = 'DSP';
      TeeMsg_GallerySignalHighLow = 'Signal high-low';
{$ENDIF}

{ TDiscreteSeries }

Procedure TSignalDiscreteSeries.PrepareCanvas(AColor:TColor);
begin
  ParentChart.Canvas.AssignVisiblePenColor(LinePen,AColor);
  ParentChart.Canvas.BackMode:=cbmTransparent;
end;

procedure TSignalDiscreteSeries.DrawAllValues;
type TChartRecord = record
        X: integer;
        Y: integer;
        Color: TColor;
        BottomPos: integer;
     end;
var a: array of TChartRecord;
    i,j: integer;
begin
  SetLength(a,LastValueIndex-FirstValueIndex+1);
  if LinePen.Visible then
  begin
       for i := 0 to Length(a)-1 do
       begin
           j := FirstValueIndex+i;
           a[i].X := CalcXPos(j);
           a[i].Y := CalcYPos(j);
           a[i].BottomPos := GetOriginPos(j);
           a[i].Color := ValueColor[j];
           if a[i-1].Color = clNone then Continue;
           PrepareCanvas(a[i].Color);
           ParentChart.Canvas.VertLine3D(a[i].X,
                                         a[i].Y,a[i].BottomPos,
                                         StartZ);
       end;
  end;
  if Pointer.Visible then
  Begin  {Dont draw the first one, if not there}
       for i := 0 to Length(a)-1 do if (a[i].Color<>clNone) then
            DrawPointer(a[i].X,a[i].Y,a[i].Color, FirstValueIndex+i);
  end;
end;

constructor TSignalDiscreteSeries.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTargetZOrder := -1;
end;

procedure TSignalDiscreteSeries.SetOrigin(const Value: Double);
begin
  SetDoubleProperty(FOrigin,Value)
end;

procedure TSignalDiscreteSeries.SetUseOrigin(const Value: Boolean);
begin
  SetBooleanProperty(FUseOrigin,Value)
end;


class Function TSignalDiscreteSeries.GetEditorClass:String;
Begin
    result:='TSignalDiscreteSeriesEditor';
End;

procedure TSignalDiscreteSeries.Assign(Source: TPersistent);
begin
      inherited;
      if Source is TSignalDiscreteSeries then
      With TSignalDiscreteSeries(Source) do
      begin
         Self.FUseOrigin :=UseYOrigin;
         Self.FOrigin    :=YOrigin;
      end
end;

function TSignalDiscreteSeries.MaxYValue: Double;
begin
  result:=inherited MaxYValue;
  if UseYOrigin and (result<YOrigin) then result:=YOrigin;
end;

function TSignalDiscreteSeries.MinYValue: Double;
begin
  result:=inherited MinYValue;
  if UseYOrigin and (result>YOrigin) then result:=YOrigin;
end;

function TSignalDiscreteSeries.GetOriginPos(ValueIndex: Integer): Integer;
begin
  if UseYOrigin then result:= CalcYPosValue(YOrigin)
                else result:= GetVertAxis.IEndPos-1;
end;

procedure TSignalDiscreteSeries.SetTargetZOrder(const Value: integer);
begin
  FTargetZOrder := Value;
end;

procedure TSignalDiscreteSeries.CalcZOrder;
begin
     if FTargetZOrder > -1 then ZOrder := FTargetZOrder
                           else inherited CalcZOrder;
end;

function TSignalDiscreteSeries.MoreSameZOrder: Boolean;
begin
     if FTargetZOrder > -1 then Result := false
                           else Result := inherited MoreSameZOrder;
end;


{ TSignalAreaSeries}

(*  function TSignalAreaSeries.GetOriginPos(ValueIndex: Integer): Integer;
  begin
     if UseYOrigin then result:= CalcYPosValue(YOrigin)
                   else result:= GetVertAxis.IEndPos-1;
  end;

  Function TSignalAreaSeries.AAVCalcYPosLeftRight(Const YLimit:Double;
                      AnotherIndex:Integer; ValueIndex:Integer):Integer;
  var tmpPredValueX : Double;
      tmpPredValueY : Double;
      tmpDif        : Double;
  begin
    tmpPredValueX:=XValues.Value[AnotherIndex];
    tmpDif:=XValues.Value[ValueIndex]-tmpPredValueX;
    With ParentChart do
    if tmpDif=0 then result:=CalcYPos(AnotherIndex)
    else
    begin
      tmpPredValueY:=YValues.Value[AnotherIndex];
      result:=CalcYPosValue( 1.0*tmpPredValueY+(YLimit-tmpPredValueX)*
                             (YValues.Value[ValueIndex]-tmpPredValueY)/tmpDif );
    end;
  end;

   Procedure TSignalAreaSeries.AAVInternalDrawArea(BrushColor:TColor;
     ValueIndexi,i:Integer; const a: array of TChartRecord; IsLastValue: boolean);

   var tmpY      : Integer;
       tmpBottom : Integer;
       tmpR      : TRect;
   Begin
     With ParentChart do
     Begin
       SetBrushCanvas(BrushColor,AreaChartBrush,SeriesColor);
       if View3D and IsLastValue and not FrontPlaneOnly then
          Canvas.RectangleZ(a[i].X,a[i].Y,a[i].BottomPos,StartZ,EndZ);
       if Stairs then
       begin
         if InvertedStairs then
         begin
           tmpY := a[i].Y;
           tmpBottom := a[i].BottomPos;
         end
         else
         begin
           tmpY := a[i-1].Y;
           tmpBottom := a[i-1].BottomPos;
         end;
         tmpR:={$IFDEF D6}Types.{$ENDIF}Rect(a[i-1].X,tmpBottom,a[i].X+1,tmpY);
         With Canvas do
         begin
           if View3D then
           begin
             RectangleWithZ(tmpR,StartZ);
             if SupportsFullRotation then RectangleWithZ(tmpR,EndZ);
           end
           else DoRectangle(tmpR);
         end;
       end
       else
       With Canvas do
       begin
         PlaneWithZ( {$IFDEF D6}Types.{$ENDIF}Point(a[i-1].X,a[i-1].BottomPos),
                     {$IFDEF D6}Types.{$ENDIF}Point(a[i-1].X,a[i-1].Y),
                     {$IFDEF D6}Types.{$ENDIF}Point(a[i].X,a[i].Y),
                     {$IFDEF D6}Types.{$ENDIF}Point(a[i].X,a[i].BottomPos),StartZ);
         if SupportsFullRotation then
            PlaneWithZ( {$IFDEF D6}Types.{$ENDIF}Point(a[i-1].X,a[i-1].BottomPos),
                        {$IFDEF D6}Types.{$ENDIF}Point(a[i].X,a[i].BottomPos),
                        {$IFDEF D6}Types.{$ENDIF}Point(a[i].X,a[i].Y),
                        {$IFDEF D6}Types.{$ENDIF}Point(a[i-1].X,a[i-1].Y),EndZ);
       end;
     end;
   end;

procedure TSignalAreaSeries.DrawPointers(const a: array of TChartRecord; k: integer);
var i: integer;
begin
     if Pointer.Visible then
     Begin  {Dont draw the first one, if not there}
         for i := k to Length(a)-1 do
         if (a[i].Color<>clNone) then
           DrawPointer(a[i].X,a[i].Y,a[i].Color, FirstValueIndex+i-k);
     end;
end;

Procedure TSignalAreaSeries.AAVDrawPoint(ValueIndex,i:Integer;
                Const a: array of TChartRecord; IsLastValue: boolean);
var tmpPoint     : TPoint;
   tmpOldP      : TPoint;
   tmpDifX      : Integer;
   P4           : TFourPoints;
   OldDarkColor : TColor;
   tmpDark3D    : Boolean;
Begin
    if (a[i].Color<>clNone) then { <-- if not null }
     With ParentChart,Canvas do
     Begin
       if View3D then
       Begin
         if DrawArea and not FrontPlaneOnly then
         Begin
           Brush.Color:=GetAreaBrushColor(a[i].Color);
           OldDarkColor:=Brush.Color;

           AssignVisiblePen(LinePen);
           if LinePen.Visible then CheckPenWidth(Pen);
           Brush.Style:=LineBrush;
           if Assigned(Self.Brush.Image.Graphic) then
              Brush.Bitmap:=Self.Brush.Image.Bitmap;

           tmpPoint.X  := a[i].X;
           tmpPoint.Y  := a[i].Y;
           tmpOldP.X   := a[i-1].X;
           tmpOldP.Y   := a[i-1].Y;
           if Stairs then
           Begin
             if InvertedStairs then
             begin
               if Dark3D then Brush.Color:=ApplyDark(Brush.Color,DarkColorQuantity);
               RectangleZ( tmpOldP.X,tmpOldP.Y, a[i].Y,StartZ,EndZ);
               if Dark3D then Brush.Color:=OldDarkColor;
               RectangleY( tmpPoint.X, tmpPoint.Y,a[i-1].X,StartZ,EndZ);
             end
             else
             begin
               RectangleY( tmpOldP.X, tmpOldP.Y, a[i].X, StartZ, EndZ);
               if Dark3D then Brush.Color:=ApplyDark(Brush.Color,DarkColorQuantity);
               RectangleZ( tmpPoint.X,tmpPoint.Y, a[i-1].Y,StartZ,EndZ);
               if Dark3D then Brush.Color:=OldDarkColor;
             end;
           end
           else
           begin
             if (LineHeight>0) and (not DrawArea) then
             begin
               P4[0]:=tmpPoint;
               P4[1]:=tmpOldP;
               P4[2].X:=tmpOldP.X;
               P4[2].Y:=tmpOldP.Y+LineHeight;
               P4[3].X:=tmpPoint.X;
               P4[3].Y:=tmpPoint.Y+LineHeight;
               PlaneFour3D(P4,StartZ,StartZ);
               if IsLastValue then
                  RectangleZ(tmpPoint.X,tmpPoint.Y,tmpPoint.Y+LineHeight,StartZ,EndZ);
             end;
             tmpDark3D := Dark3D and (not SupportsFullRotation);
             if tmpDark3D then
             begin
               tmpDifX:=tmpPoint.X-tmpOldP.X;
               if (tmpDifX<>0) and
                  (tmpDark3DRatio<>0) and
                  ((tmpOldP.Y-tmpPoint.Y)/tmpDifX > tmpDark3DRatio) then
               begin
                 Brush.Color := ApplyDark(Brush.Color,DarkColorQuantity);
                 if (LineHeight>0) and (not DrawArea) then {special case}
                 begin
                   Inc(tmpPoint.Y,LineHeight);
                   Inc(tmpOldP.Y,LineHeight);
                 end;
               end;
             end;
             if Monochrome then Brush.Color:=clWhite;
             Plane3D(tmpPoint,tmpOldP,StartZ,EndZ);
             if tmpDark3D then Brush.Color:=OldDarkColor;
           end;
         end;
       end;
       if DrawArea then
       Begin
         Brush.Color:=GetAreaBrushColor(a[i].Color);
         if (AreaLinesPen.Color=clTeeColor) or (not AreaLinesPen.Visible) then
            AssignVisiblePenColor(AreaLinesPen,a[i].Color)
         else
            AssignVisiblePen(AreaLinesPen);
         AAVInternalDrawArea(Brush.Color,ValueIndex,i,a,IsLastValue);
       end;
     end;
   end;

Procedure TSignalAreaSeries.DrawLines(Const a: array of TChartRecord);
var i: integer;
begin
     if not ParentChart.View3D then
     begin
         if LinePen.Visible then
         Begin
             ParentChart.Canvas.AssignVisiblePen(LinePen);
             with ParentChart,Canvas do
             if ColorEachLine then
             begin
                 for i := 1 to Length(a)-1 do
                 if (a[i].Color<>clNone) then
                 begin
//                    if FDrawLine then LinePrepareCanvas(Canvas,a[i].Color);
                    if Stairs then
                    begin
                         if InvertedStairs then DoVertLine(a[i-1].X,a[i-1].Y,a[i].Y)
                          else DoHorizLine(a[i-1].X,a[i].X,a[i-1].Y);
                         LineTo(a[i].X,a[i].Y);
                    end else Line(a[i-1].X,a[i-1].Y,a[i].X,a[i].Y);
                 end;
             end else
             begin
//                 if fDrawLine then LinePrepareCanvas(Canvas,SeriesColor);
                 for i := 1 to Length(a)-1 do
                 if (a[i].Color<>clNone) then
                 begin
                    if Stairs then
                    begin
                         if InvertedStairs then DoVertLine(a[i-1].X,a[i-1].Y,a[i].Y)
                          else DoHorizLine(a[i-1].X,a[i].X,a[i-1].Y);
                         LineTo(a[i].X,a[i].Y);
                    end else Line(a[i-1].X,a[i-1].Y,a[i].X,a[i].Y);
                 end;
             end;
         end;
     end else
     begin
           if DrawArea and LinePen.Visible and not Stairs then
           begin
               with ParentChart,Canvas do
               begin
               AssignVisiblePen(LinePen);
               for i := 1 to Length(a)-1 do LineWithZ(a[i-1].X,a[i-1].Y,a[i].X,a[i].Y,StartZ);
               end;
           end;
           {Missing:
           - Line series with front panel only. (Just line in 3D)
           - Draw lines over steps, (Need DoHorizLineWithZ)}
     end;
end;

procedure TSignalAreaSeries.DrawAllValues;
var a: array of TChartRecord;
    i,j,k: integer;
begin
     k := Byte(FirstValueIndex > 0); {Previous point outside left}
     SetLength(a,LastValueIndex-FirstValueIndex+1+k);
     with ParentChart.Canvas do
     begin
       if (k = 0) then
       begin
           a[0].X := CalcXPos(FirstValueIndex);
           a[0].Y := CalcYPos(FirstValueIndex);
           a[0].BottomPos := GetOriginPos(FirstValueIndex);
           a[0].Color := ValueColor[FirstValueIndex]; {TmpColor}
       end else
       begin
           if DrawArea then
           begin
             a[0].X := CalcXPos(Pred(FirstValueIndex));
             a[0].Y:= CalcYPos(Pred(FirstValueIndex));
             a[0].BottomPos:=GetOriginPos(Pred(FirstValueIndex));
           end
           else
           begin
             if GetHorizAxis.Inverted then a[0].X:=ParentChart.ChartRect.Right
                                      else a[0].X:=ParentChart.ChartRect.Left;
             if Stairs Then a[0].Y:=CalcYPos(Pred(FirstValueIndex))
                       else a[0].Y:=AAVCalcYPosLeftRight(XScreenToValue(a[0].X),Pred(FirstValueIndex),FirstValueIndex);
           end;
           a[0].Color := ValueColor[FirstValueIndex-1]; {TmpColor}
       end;
       for i := 1 to Length(a)-1 do
       begin
           j := FirstValueIndex-k+i;
           a[i].X := CalcXPos(j);
           a[i].Y := CalcYPos(j);
           a[i].BottomPos := GetOriginPos(j);
           a[i].Color := ValueColor[j];
           if a[i-1].Color = clNone then a[i-1] := a[i];
       end;

       if Dark3D then
       With ParentChart do
       begin
           if SeriesWidth3D<>0 then tmpDark3DRatio:=Abs(SeriesHeight3D/SeriesWidth3D)
           else tmpDark3DRatio:=1;
       end;
       if SupportsFullRotation and DrawArea and ParentChart.View3D then RectangleZ(a[0].X,a[0].Y,a[0].BottomPos,StartZ,EndZ); {obsolete}
       {Handle all other values, except last}
       if ParentChart.View3D or DrawArea then
       for i := 1 to Length(a) - 1 do
       begin
           Pen.Color:=clBlack;
           Brush.Color:= a[i].Color;
           AAVDrawPoint(i+FirstValueIndex-1,i,a,False);
       end;
       {Handle last value}
       Pen.Color:=clBlack;
       Brush.Color:= a[Length(a)-1].Color;
       AAVDrawPoint(LastValueIndex,Length(a)-1,a,True);
       DrawLines(a);
       DrawPointers(a,k);
  end;
end;

class Function TSignalAreaSeries.GetEditorClass:String;
Begin
  result:='TSignalAreaSeriesEditor'; { <-- dont translate ! }
end;

procedure TSignalAreaSeries.SetFrontPlaneOnly(const Value: boolean);
begin
  SetBooleanProperty(FFrontPanelOnly,Value);
end;

function TSignalAreaSeries.MoreSameZOrder: Boolean;
begin
     Result := false;
end;*)


{ TSignalHighLowSeries }

Constructor TSignalHighLowSeries.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  HighBrush.Style:=bsSolid;
  SeriesMode := ssmHighLow;
end;

function TSignalHighLowSeries.MoreSameZOrder: Boolean;
begin
     Result := false;
end;

procedure SetPoints(const x,y: array of integer; var Points: array of TPoint;  Index: integer = 0; Len: integer = -1);
var i: integer;
    px,pxy,py: PInteger;
begin
     if Len = -1 then Len := Length(Points)-Index;
     if (Index+Len) > Length(Points) then raise Exception.Create('(Index+Len)*2 > Length(xy)');
     i := 0;
     px := @x[0];
     py := @y[0];
     pxy := @Points[Index];
     while i < Len do
     begin
          pxy^ := px^;
          Inc(px);
          Inc(pxy);
          pxy^ := py^;
          Inc(py);
          Inc(pxy);
          Inc(i);
     end;
end;

procedure SetPointsReverse(const x,y: array of integer; var Points: array of TPoint;  Index: integer = 0; Len: integer = -1);
var i: integer;
    px,pxy,py: PInteger;
begin
     if Len = -1 then Len := Length(Points)-Index;
     if (Index+Len) > Length(Points) then raise Exception.Create('(Index+Len) > Length(Points)');
     i := 0;
     px := @x[0];
     py := @y[0];
     pxy := @Points[Index+Len-1];
     Inc(pxy);
     while i < Len do
     begin
          pxy^ := py^;
          Inc(py);
          Dec(pxy);
          pxy^ := px^;
          Inc(px);
          Dec(pxy);
          Inc(i);
     end;
end;

(*procedure TSignalHighLowSeries.DrawAllValuesVec;
{var x,y,z: array of integer;
    Points: array of TPoint;
    Index,Len,i: integer;
    par: cardinal;}
begin
(*   with ParentChart,Canvas do
   begin
{       par := GetTickCount;
       for i := 0 to 100-1 do
       begin}
       Index := FirstValueIndex;
       Len := LastValueIndex-FirstValueIndex+1;
       SetLength(x,Len);
       SetLength(y,Len);
       SetLength(z,Len);
       CalcXPosValueVec(XValues.Value,x,Index);
       CalcYPosValueVec(YValues.Value,y,Index);
       if LowValues.Count = 0 then
       begin
           SetLength(LowValues.Value,Length(x));
           FillChar(LowValues.Value[0],SizeOf(Double)*Length(x),0);
       end;
       CalcYPosValueVec(LowValues.Value,z,Index);

       AssignBrush(Self.HighBrush,ValueColor[FirstValueIndex+1]);
       if (Brush.Style <> bsClear) then Pen.Style := psClear;

       SetLength(Points,Length(x)*2);
       SetPoints(x,y,Points,0,Length(x));
       SetPoints(x,z,Points,Length(x),Length(x));
       if View3D then Calc3DPosVec(Points,StartZ);
       PolygonArea(Points,PPoint(@Points[Length(x)])^,0,Length(x));

       AssignVisiblePen(HighPen);
       PolylineVec(Points,0,Length(x));

       AssignVisiblePen(LowPen);
       PolylineVec(Points,Length(x),Length(x));
{       end;
       par := GetTickCount-par;
       ShowMessage(IntToStr(par));}
  end;
end;   *)

Function TeePoint(aX,aY:Integer):TPoint;
begin
  with result do
  begin
    X:=aX;
    Y:=aY;
  end;
end;

Procedure PolygonArea(var PointsHigh,PointsLow: array of TPoint; Index,Len: integer; Canvas: TCanvas3D);
type PPoints = ^TPoint;
var i: integer;
    IPoints: TFourPoints;
begin
{$R-}
  if Len = -1 then Len := Length(PointsHigh)-Index;
  for i := Index to Index+Len-2 do
  begin  {Draw by recursively using the previous point.}
      IPoints[0] := PointsHigh[i];
      IPoints[1] := PointsHigh[i+1];
      IPoints[2] := PointsLow[i+1];
      IPoints[3] := PointsLow[i];
     {$IFNDEF CLX}
      Windows.Polygon(Canvas.Handle, PPoints(@IPoints)^, 4);
     {$ELSE}
      FCanvas.Polygon(IPoints);
     {$ENDIF}
  end;
end;

procedure PolyLineVec(var Points: array of TPoint; Index,Len: integer; Canvas: TCanvas3D);
type PPoints = ^TPoint;
begin
    if Len = -1 then Len := Length(Points)-Index;
    if (Len+Index) > Length(Points) then raise Exception.Create('(Len+Index) > Length(Points)');
    {$IFNDEF CLX}
    Windows.Polyline(Canvas.Handle, PPoints(@Points[Index])^, Len);
    {$ELSE}
    {Problem here for Index > 0};
    FCanvas.PolyLine(Slice(Points,Len));
    {$ENDIF}
end;

Function TSignalHighLowSeries.MaxYValue:Double;
Begin
   if SeriesMode = ssmLine then Result := HighValues.MaxValue
                           else Result := inherited MaxYValue;
End;

Function TSignalHighLowSeries.MinYValue:Double;
Begin
   if SeriesMode = ssmLine then Result := HighValues.MinValue
                           else Result := inherited MinYValue;
End;

procedure TSignalHighLowSeries.DrawAllValues;
var i,Index,Len: integer;
    Points: array of TPoint;
begin
   case SeriesMode of
   ssmHighLow:
   with ParentChart,Canvas do
   begin
       Index := FirstValueIndex;
       Len := LastValueIndex-FirstValueIndex+1;
       SetLength(Points,Len*2);
       if LowValues.Count <> HighValues.Count then
       begin
            SetLength(LowValues.Value,HighValues.Count);
            FillChar(LowValues.Value[0],SizeOf(Double)*HighValues.Count,0);
       end;
       for i := 0 to Len-1 do
       begin
            Points[i].X := CalcXPosValue(XValues.Value[Index+i]);
            Points[i].Y := CalcYPosValue(YValues.Value[Index+i]);
            Points[i+Len].X := Points[i].X;
            Points[i+Len].Y := CalcYPosValue(LowValues.Value[Index+i]);
            if View3D then
            begin
                 Points[i] := Calculate3DPosition(Points[i].X,Points[i].Y,StartZ);
                 Points[i+Len] := Calculate3DPosition(Points[i+Len].X,Points[i+Len].Y,StartZ);
            end;
       end;
       AssignBrush(Self.HighBrush,ValueColor[FirstValueIndex+1]);
       if (Brush.Style <> bsClear) then Pen.Style := psClear;
       PolygonArea(Points,PPoint(@Points[Len])^,0,Len, Canvas);

       AssignVisiblePen(HighPen);
       PolylineVec(Points,0,Len,Canvas);

       AssignVisiblePen(LowPen);
       PolylineVec(Points,Len,Len,Canvas);
   end;
   ssmLine:  //do simple fast line
   with ParentChart,Canvas do
   begin
       Index := FirstValueIndex;
       Len := LastValueIndex-FirstValueIndex+1;
       SetLength(Points,Len);
       if LowValues.Count <> HighValues.Count then
       begin //Must be here for compatibility with streaming...
            SetLength(LowValues.Value,HighValues.Count);
            FillChar(LowValues.Value[0],SizeOf(Double)*HighValues.Count,0);
       end;
       for i := 0 to Len-1 do
       begin
            Points[i].X := CalcXPosValue(XValues.Value[Index+i]);
            Points[i].Y := CalcYPosValue(YValues.Value[Index+i]);
            if View3D then Points[i] := Calculate3DPosition(Points[i].X,Points[i].Y,StartZ);
       end;
       Self.Pen.Style := psSolid;
       AssignVisiblePen(Self.Pen);
       PolylineVec(Points,0,Len,Canvas);
   end;
   end;
end;

procedure TSignalHighLowSeries.SetSeriesMode(
  const Value: TSignalSeriesMode);
begin
  FSeriesMode := Value;
end;

procedure TSignalHighLowSeries.Assign(Source: TPersistent);
begin
  inherited;
  SeriesMode := TSignalHighLowSeries(Source).SeriesMode;
end;

initialization
 {$IFDEF TEE5}
  TeeMsg_GallerySignalArea := 'Signal area';
  TeeMsg_GallerySignalPoint := 'Signal point';
  TeeMsg_GalleryDSP := 'DSP';
  TeeMsg_GallerySignalHighLow := 'Signal high-low';
 {$ENDIF}

(*  RegisterTeeSeries(TSignalAreaSeries,
                    {$IFDEF TEE5}@{$ENDIF}TeeMsg_GallerySignalArea,
                    {$IFDEF TEE5}@{$ENDIF}TeeMsg_GalleryDSP,2); *)
  RegisterTeeSeries(TSignalDiscreteSeries,
                    {$IFDEF TEE5}@{$ENDIF}TeeMsg_GallerySignalPoint,
                    {$IFDEF TEE5}@{$ENDIF}TeeMsg_GalleryDSP,2);

  RegisterTeeSeries(TSignalHighLowSeries,
                     {$IFDEF TEE5}@{$ENDIF}TeeMsg_GallerySignalHighLow,
                     {$IFDEF TEE5}@{$ENDIF}TeeMsg_GalleryDSP,2);                    

finalization
 {$IFDEF TEE5}
  TeeMsg_GallerySignalArea := '';
  TeeMsg_GallerySignalPoint := '';
  TeeMsg_GalleryDSP := '';
  TeeMsg_GallerySignalHighLow := '';
 {$ENDIF}
  UnRegisterTeeSeries([TSignalDiscreteSeries,TSignalHighLowSeries]);

end.
